@@ -49,7 +49,6 @@
 
 #if HAVE_RSVG
 #include <librsvg/rsvg.h>
-#include <librsvg/librsvg-features.h>
 #endif
 
 #define EOG_CSS_FILE_PATH EOG_DATA_DIR G_DIR_SEPARATOR_S "eog.css"
@@ -144,9 +143,6 @@ main (int argc, char **argv)
 
 #ifdef HAVE_EXEMPI
  	xmp_init();
-#endif
-#if defined (HAVE_RSVG) && !LIBRSVG_CHECK_VERSION(2, 35, 1)
-	rsvg_init();
 #endif
 	eog_debug_init ();
 	eog_job_queue_init ();
@@ -194,9 +190,6 @@ main (int argc, char **argv)
   	if (startup_files)
 		g_strfreev (startup_files);
 
-#if defined (HAVE_RSVG) && !LIBRSVG_CHECK_VERSION(2, 35, 1)
-	rsvg_term();
-#endif
 #ifdef HAVE_EXEMPI
 	xmp_terminate();
 #endif
